-- TE4 - T-Engine 4
-- Copyright (C) 2009, 2010, 2011, 2012 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

local _M = loadPrevious(...)

local base_resolveLevelTalents = _M.resolveLevelTalents

--- Resolve leveling talents
--I5 Open up increased talent levels for normal monsters
function _M:resolveLevelTalents()
	if not self.start_level or not self._levelup_talents then return end

--I5
	local maxfact = 1  --I5 important balancing parameter for high level normal actors, higher value increases maximum talent level
	if game.zone.short_name == "infinite-dungeon" then maxfact=math.max(maxfact,self.level/50) end 

	for tid, info in pairs(self._levelup_talents) do

--I5		print("[TALENT COUNT]", self.name, "level",self.level, "uid:",self.uid, "maxfact:",maxfact)

		if not info.max or (self.talents[tid] or 0) < math.floor(info.max*maxfact) then
			local last = info.last or self.start_level

--I5	print("[TALENT COUNT]",self.name,"uid:",self.uid,"checking talent:", tid,"base:",info.base,"every:",info.every,"max:",info.max or 0)
			if self.level - last >= info.every then
				self:learnTalent(tid, true)
				info.last = self.level

--I5			print("[TALENT COUNT]",self.name,"uid:",self.uid,"learned talent:",tid, "in",game.zone.name)

			end
		end
	end
end

--I5 Updated to allow for partial talent redefinitions (merge)
--I5 t.name (Talent name) and t.type (Talent type) are only required fields in a redefinition
--- Defines one talent
-- Static!
function _M:newTalent(t)
	assert(t.name, "no talent name")
	assert(t.type, "no or unknown talent type")
	if type(t.type) == "string" then t.type = {t.type, 1} end
	if not t.type[2] then t.type[2] = 1 end
	t.short_name = t.short_name or t.name
	t.short_name = t.short_name:upper():gsub("[ ']", "_")
--I5	t.mode = t.mode or "activated"
--I5	t.points = t.points or 1
--I5	assert(t.mode == "activated" or t.mode == "sustained" or t.mode == "passive", "wrong talent mode, requires either 'activated' or 'sustained'")
--I5	assert(t.info, "no talent info")

	-- Can pass a string, make it into a function
	if type(t.info) == "string" then
		local infostr = t.info
		t.info = function() return infostr end
	end
	-- Remove line stat with tabs to be cleaner ..
	if t.info then
		local info = t.info
		t.info = function(self, t) return info(self, t):gsub("\n\t+", "\n") end
	end
	
	t.id = "T_"..t.short_name
--I5	self.talents_def[t.id] = t
--I5	assert(not self[t.id], "talent already exists with id T_"..t.short_name)
	if self[t.id] then -- I5 Merge tables -- i.e. overwrite previous values with new ones
		print("[TALENT] Talent "..t.short_name.." already exists.  Updating with new definition information.")
		
		--I5 Update talent type tables if type has changed
		if self.talents_def[t.id].type[1] ~= t.type[1] then
			print("[TALENT]   Updating type tables for "..t.id)
			table.removeFromList(self.talents_types_def[self:getTalentFromId(t.id).type[1]].talents,self.talents_def[t.id])  --I5 Unregister from old type table
			table.insert(self.talents_types_def[t.type[1]].talents, t) -- I5 Register with new type table
		end
		--I5 merge the talent data
		table.merge(self.talents_def[t.id],t)
		
--I5		self[t.id]=t.id
	else
		self.talents_def[t.id] = t
		self[t.id] = t.id
		-- Register in the type table
		table.insert(self.talents_types_def[t.type[1]].talents, t)
	end

--I5	self[t.id] = t.id
	print("[TALENT]", t.name, t.short_name, t.id)
--I5	assert(self.talents_def[t.id].type, "no or unknown talent type") --I5
	t.mode = t.mode or "activated"
	assert(t.mode == "activated" or t.mode == "sustained" or t.mode == "passive", "wrong talent mode, requires either 'activated' or 'sustained'") --I5
	t.points = t.points or 1
	assert(self.talents_def[t.id].info, "no talent info") --I5
	
	-- Register in the type table
--I5	table.insert(self.talents_types_def[t.type[1]].talents, t)
end

return _M