-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011, 2012 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

--require "engine.class"
--require "engine.Actor"
--require "engine.Autolevel"
--require "engine.interface.ActorInventory"
require "engine.interface.ActorTemporaryEffects"
--require "mod.class.interface.ActorLife"
--require "engine.interface.ActorProject"
--require "engine.interface.ActorLevel"
--require "engine.interface.ActorStats"
--require "engine.interface.ActorTalents"
--require "engine.interface.ActorResource"
--require "engine.interface.BloodyDeath"
--require "engine.interface.ActorFOV"
--require "mod.class.interface.ActorPartyQuest"
--require "mod.class.interface.Combat"
--require "mod.class.interface.Archery"
--require "mod.class.interface.ActorInscriptions"
local Faction = require "engine.Faction"
local Dialog = require "engine.ui.Dialog"
local Map = require "engine.Map"
local DamageType = require "engine.DamageType"

local _M = loadPrevious(...)

-- Tells on_set_temporary_effect() what save to use for a given effect type
local save_for_effects = {
	magical = "combatSpellResist",
	mental = "combatMentalResist",
	physical = "combatPhysicalResist",
}

--Choose what to print in verbose log: LOS to player(normal) or all for cheat mode
local verbosable = function(who)
	return game.log and config.settings.tome.game_mechanics.verbose_game_log and (config.settings.cheat or who:hasLOS(game.player.x,game.player.y))
end

--- Adjust temporary effects for saves, immunities, etc...
-- @param eff_id Effect ID
-- @param e  Effect definition
-- @param p  Effect parameter table
function _M:on_set_temporary_effectAlt(eff_id, e, p)
	if p.apply_power and (save_for_effects[e.type] or p.apply_save) then
		local save = 0
		p.maximum = p.dur
		p.minimum = p.min_dur or 0 --Default minimum duration is 0. Can specify something else by putting min_dur=foo in p when calling setEffect()
		save = self[p.apply_save or save_for_effects[e.type]](self)
		--local duration = p.maximum - math.max(0, math.floor((save - p.apply_power) / 5))
		--local duration = p.maximum - math.max(0, (math.floor(save/5) - math.floor(p.apply_power/5)))
		
		local percentage = 0
		if config.settings.tome.game_mechanics.norm_saves then
			local savex, savechance = self:checkHitOld(save, p.apply_power) --I5 get save chance
--			savex = rng.normalFloat(50,25) -- I5 normal distribution about 50 with SDev of 25
--			savex = rng.normalFloat(savechance,25) -- % reduction, get a normal distribution of reduction chances
--			percentage = util.bound(1-savex/100,0,2)
			savex = rng.normalFloat((100-savechance)*2,50) -- fraction duration, normal distribution weighted to be small at even odds
			percentage = util.bound(savex/100,0,2)
			
--			if config.settings.tome.game_mechanics.verbose_game_log and game.log then
			if verbosable(self) then
				local oldpercentage = 1 - ((save - p.apply_power)/20)
				game.log("#LIGHT_RED#Effect duration: Save: %d vs Power %d (%0.1f%% save chance) gave %0.1f%% duration (vs old %0.f%%)",save, p.apply_power,savechance,percentage*100, oldpercentage*100)
			end
		else
			percentage = 1 - ((save - p.apply_power)/20)
		end
		
		local duration = math.min(p.maximum, math.ceil(p.maximum * percentage))
		p.dur = util.bound(duration, p.minimum or 0, p.maximum)
		p.amount_decreased = p.maximum - p.dur
		local save_type = nil

		if p.apply_save then save_type = p.apply_save else save_type = save_for_effects[e.type] end
		if save_type == "combatPhysicalResist" then p.save_string = "Physical save"
		elseif save_type == "combatMentalResist" then p.save_string = "Mental save"
		elseif save_type == "combatSpellResist" then p.save_string = "Spell save"
		end

		if not p.no_ct_effect and not e.no_ct_effect and e.status == "detrimental" then self:crossTierEffect(eff_id, p.apply_power, p.apply_save or save_for_effects[e.type]) end
		p.total_dur = p.dur

		if e.status == "detrimental" and self:checkHit(save, p.apply_power, 0, 95) and p.dur > 0 then
			game.logSeen(self, "#ORANGE#%s shrugs off the effect '%s'!", self.name:capitalize(), e.desc)
			p.dur = 0
		end

		p.apply_power = nil
	end

	if e.status == "detrimental" and self:knowTalent(self.T_RESILIENT_BONES) then
		local durred = self:callTalent(self.T_RESILIENT_BONES,"durresist") --for I500 talent change
		if durred then
			p.dur = math.ceil(p.dur * (1 - durred))
		else
			p.dur = math.ceil(p.dur * (1 - util.bound(self:getTalentLevel(self.T_RESILIENT_BONES) / 12, 0, 1)))
		end
		
	end
	if e.type ~= "other" and self:attr("reduce_status_effects_time") then
		local power = util.bound(self.reduce_status_effects_time, 0, 100)
		p.dur = math.ceil(p.dur * (1 - (power/100)))
	end
	if self:knowTalent(self.T_VITALITY) and e.status == "detrimental" and (e.subtype.wound or e.subtype.poison or e.subtype.disease) then
		local t = self:getTalentFromId(self.T_VITALITY)
		p.dur = math.ceil(p.dur * (1 - util.bound(t.getWoundReduction(self, t), 0, 1)))
	end
	if self:hasEffect(self.EFF_HAUNTED) and e.subtype and e.subtype.fear then
		local e = self.tempeffect_def[self.EFF_HAUNTED]
		e.on_setFearEffect(self, e)
	end
	if e.status == "detrimental" and e.type ~= "other" and self:attr("negative_status_effect_immune") then
		p.dur = 0
	end
	if e.status == "detrimental" and e.type == "mental" and self:attr("mental_negative_status_effect_immune") and not e.subtype["cross tier"] then
		p.dur = 0
		self:attr("mental_negative_status_effect_immune", -1)
		if not self:attr("mental_negative_status_effect_immune") then self:removeEffect(self.EFF_CLEAR_MIND) end
	end
	if e.status == "detrimental" and e.type == "mental" and self:knowTalent(self.T_UNBREAKABLE_WILL) and not e.subtype["cross tier"] then
		if self:triggerTalent(self.T_UNBREAKABLE_WILL) then p.dur = 0 end
	end
	if e.status == "detrimental" and e.type == "physical" and self:attr("physical_negative_status_effect_immune") and not e.subtype["cross tier"] then
		p.dur = 0
	end
	if e.status == "detrimental" and e.type == "spell" and self:attr("spell_negative_status_effect_immune") and not e.subtype["cross tier"] then
		p.dur = 0
	end
	if self:attr("status_effect_immune") then
		p.dur = 0
	end

	if p.dur > 0 and not e.subtype["cross tier"] and e.status == "detrimental" and e.type == "physical" and self:knowTalent(self.T_SPINE_OF_THE_WORLD) then
		self:triggerTalent(self.T_SPINE_OF_THE_WORLD)
	end

	if self.player then
		p.__set_time = core.game.getTime()
	end
end

return _M