-- ToME - Tales of Maj'Eyal
-- Copyright (C) 2009, 2010, 2011, 2012 Nicolas Casalini
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Nicolas Casalini "DarkGod"
-- darkgod@te4.org

require "engine.class"
local Dialog = require "engine.ui.Dialog"
local TreeList = require "engine.ui.TreeList"
local Textzone = require "engine.ui.Textzone"
local Separator = require "engine.ui.Separator"
local GetQuantity = require "engine.dialogs.GetQuantity"

module(..., package.seeall, class.inherit(Dialog))

function _M:init()
	Dialog.init(self, "Optional Game Mechanics", game.w * 0.8, game.h * 0.8)

	self.c_desc = Textzone.new{width=math.floor(self.iw / 2 - 10), height=self.ih, text=""}

	self:generateList()

	self.c_list = TreeList.new{width=math.floor(self.iw / 2 - 10), height=self.ih - 10, scrollbar=true, columns={
		{width=60, display_prop="name"},
		{width=40, display_prop="status"},
	}, tree=self.list, fct=function(item) end, select=function(item, sel) self:select(item) end}

	self:loadUI{
		{left=0, top=0, ui=self.c_list},
		{right=0, top=0, ui=self.c_desc},
		{hcenter=0, top=5, ui=Separator.new{dir="horizontal", size=self.ih - 10}},
	}
	self:setFocus(self.c_list)
	self:setupUI()

	self.key:addBinds{
		EXIT = function() game:unregisterDialog(self) end,
	}
end

function _M:select(item)
	if item and self.uis[2] then
		self.uis[2].ui = item.zone
	end
end

local saveSettings = function()
	game:saveSettings("tome.game_mechanics", ("tome.game_mechanics={norm_saves = %s,\n"):format(tostring(config.settings.tome.game_mechanics.norm_saves))..
	("verbose_game_log = %s,\n"):format(tostring(config.settings.tome.game_mechanics.verbose_game_log))..
	("limited_evasion = %s,\n"):format(tostring(config.settings.tome.game_mechanics.limited_evasion))..
	("smooth_hit_factor = %s}"):format(tostring(config.settings.tome.game_mechanics.smooth_hit_factor))
	
	)
	
--	game:saveSettings("tome.game_mechanics", ("{verbose_game_log = %s\n}"):format(tostring(config.settings.tome.verbose_game_log)))
end

function _M:generateList()
	-- Makes up the list
--	local gamemech=config.settings.tome.game_mechanics
	config.settings.tome.game_mechanics = config.settings.tome.game_mechanics or {}
	config.settings.tome.game_mechanics.smooth_hit_factor = config.settings.tome.game_mechanics.smooth_hit_factor or 0
	local list = {}
	local i = 0

	local zone = Textzone.new{width=self.c_desc.w, height=self.c_desc.h, text=string.toTString"When specified, enables smoother hit probability calculations for melee and spells.  Depending on the tuning factor (default 10 for melee = 20 for spells and most other status effects), improves the chance to hit or miss for large differences in accuracy/defense or power/saves.  The chance to hit becomes less sensitive to differences as the factor increases.#WHITE#"}
	list[#list+1] = { zone=zone, name=string.toTString"#GOLD##{bold}#Smoothed Hit Chances#WHITE##{normal}#", status=function(item)
--		return tostring(config.settings.tome.game_mechanics.smooth_hit_factor)
		return tostring(config.settings.tome.game_mechanics.smooth_hit_factor and config.settings.tome.game_mechanics.smooth_hit_factor>0 and config.settings.tome.game_mechanics.smooth_hit_factor or "disabled")
	end, fct=function(item)
		game:registerDialog(GetQuantity.new("Smoothed Hit factor", "5-100 (0 disables)", config.settings.tome.game_mechanics.smooth_hit_factor, 100, function(qty)
			qty = util.bound(qty, 0, 100)
			game:saveSettings("tome.game_mechanics", ("tome.game.mechanics.smooth_hit_factor = %d\n"):format(qty))
			config.settings.tome.game_mechanics.smooth_hit_factor = qty ~= 0 and util.bound(qty,5,100) or 0
			saveSettings()
			self.c_list:drawItem(item)
		end, 0))
	end,}

	local zone = Textzone.new{width=self.c_desc.w, height=self.c_desc.h, text=string.toTString"Limits the chance to evade melee attakcs by the Evasion and Duck and Dodge skills to 50% or less.  These talents award additional defense to compensate.  This is a nerf to the evasion mechanic to prevent higher level npcs and players from being impossible to hit with melee attacks.#WHITE#"}
	list[#list+1] = { zone=zone, name=string.toTString"#GOLD##{bold}#Limited Evasion#WHITE##{normal}#", status=function(item)
		return tostring(config.settings.tome.game_mechanics.limited_evasion and "enabled" or "disabled")
	end, fct=function(item)
		config.settings.tome.game_mechanics.limited_evasion = not config.settings.tome.game_mechanics.limited_evasion
		saveSettings()
		self.c_list:drawItem(item)
	end,}
	
	local zone = Textzone.new{width=self.c_desc.w, height=self.c_desc.h, text=string.toTString"Improves the effectiveness of saving throws by making reduced duration on detrimental effects possible if the save is less than the power of the effect.  This makes saves more likely for both players and npcs.  Reduced duration depends directly on the difference between the save bonus and the power of the effect.#WHITE#"}
	list[#list+1] = { zone=zone, name=string.toTString"#GOLD##{bold}#Normalized Saves#WHITE##{normal}#", status=function(item)
		return tostring(config.settings.tome.game_mechanics.norm_saves and "enabled" or "disabled")
		end, fct=function(item)
		config.settings.tome.game_mechanics.norm_saves = not config.settings.tome.game_mechanics.norm_saves
--		game:saveSettings("tome.game_mechanics.norm_saves", ("tome.game_mechanics.normalized_saves = %s\n"):format(tostring(config.settings.tome.norm_saves)))
		saveSettings()
		self.c_list:drawItem(item)
	end,}
	
	local zone = Textzone.new{width=self.c_desc.w, height=self.c_desc.h, text=string.toTString"Causes enabled optional game mechanics to produce extra output in the game log showing (where applicable) their effects as compared to the default game mechanics.#WHITE#"}
	list[#list+1] = { zone=zone, name=string.toTString"#GOLD##{bold}#Verbose game log#WHITE##{normal}#", status=function(item)
		return tostring(config.settings.tome.game_mechanics.verbose_game_log and "enabled" or "disabled")
	end, fct=function(item)
		config.settings.tome.game_mechanics.verbose_game_log = not config.settings.tome.game_mechanics.verbose_game_log
		saveSettings()
		self.c_list:drawItem(item)
	end,}

	self.list = list
end

